/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
package com.ebmwebsourcing.easycommons.stream;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.junit.Test;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;

import com.ebmwebsourcing.easycommons.io.IOHelper;

/**
 * @author ofabre
 * 
 */
public class StreamHelperTest {

    @Test
    public void testCopyByteArrayStreams() {
        byte[] testByteArray = "foo".getBytes();
        ByteArrayInputStream inputStream = new ByteArrayInputStream(testByteArray);
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            StreamHelper.copy(inputStream, outputStream);
            assertArrayEquals(testByteArray, outputStream.toByteArray());
        } finally {
            IOHelper.close(inputStream);
            IOHelper.close(outputStream);
        }
    }

    @Test
    public void testCopyBuffuredStreams() {
        byte[] testByteArray = "foo".getBytes();
        BufferedInputStream bufInputStream = new BufferedInputStream(new ByteArrayInputStream(
                testByteArray));
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        BufferedOutputStream bufOutputStream = new BufferedOutputStream(outputStream);
        try {
            StreamHelper.copy(bufInputStream, bufOutputStream);
            assertArrayEquals(testByteArray, outputStream.toByteArray());
        } finally {
            IOHelper.close(bufInputStream);
            IOHelper.close(bufOutputStream);
        }
    }

    @Test
    public void testCopyBuffuredInputStreamsToByteArrayOutputStream() {
        byte[] testByteArray = "foo".getBytes();
        BufferedInputStream bufInputStream = new BufferedInputStream(new ByteArrayInputStream(
                testByteArray));
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            StreamHelper.copy(bufInputStream, outputStream);
            assertArrayEquals(testByteArray, outputStream.toByteArray());
        } finally {
            IOHelper.close(bufInputStream);
            IOHelper.close(outputStream);
        }
    }

    @Test
    public void testCopyByteArrayInputStreamsToBufferedOutputStream() {
        byte[] testByteArray = "foo".getBytes();
        ByteArrayInputStream inputStream = new ByteArrayInputStream(testByteArray);
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        BufferedOutputStream bufOutputStream = new BufferedOutputStream(outputStream);
        try {
            StreamHelper.copy(inputStream, bufOutputStream);
            assertArrayEquals(testByteArray, outputStream.toByteArray());
        } finally {
            IOHelper.close(inputStream);
            IOHelper.close(bufOutputStream);
        }
    }

    @Test
    public void testGetBytesFromByteArrayInputStreams() {
        byte[] testByteArray = "foo".getBytes();
        ByteArrayInputStream inputStream = new ByteArrayInputStream(testByteArray);
        try {
            assertArrayEquals(testByteArray, StreamHelper.getBytes(inputStream));
        } finally {
            IOHelper.close(inputStream);
        }
    }

    @Test
    public void testGetBytesFromBufferedInputStreams() {
        byte[] testByteArray = "foo".getBytes();
        BufferedInputStream bufInputStream = new BufferedInputStream(new ByteArrayInputStream(
                testByteArray));
        try {
            assertArrayEquals(testByteArray, StreamHelper.getBytes(bufInputStream));
        } finally {
            IOHelper.close(bufInputStream);
        }
    }

    @Test
    public void testToString() throws IOException {
        StringBuilder bigStringBuilder = new StringBuilder();
        for (int i = 0; i < 100; i++) {
            bigStringBuilder.append("ALJAFMKLJAMLFJAMLJFALMJFA \t MLJFMAJFMAJFMJ \n");
        }
        String originalString = bigStringBuilder.toString();
        InputStream is = new ByteArrayInputStream(originalString.getBytes());
        assertEquals(originalString,StreamHelper.getString(is));
    }
}
